﻿using System;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Runtime.InteropServices;
using System.Windows.Interop;

namespace icon_cs
{
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
    public struct SHFILEINFOW
    {
        public IntPtr hIcon;
        public int iIcon;
        public uint dwAttributes;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
        public string szDisplayName;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string szTypeName;
    }

    public enum ShellFileInfoFlags : uint
    {
        SHGFI_ICON = 0x000000100,
        SHGFI_DISPLAYNAME = 0x000000200,
        SHGFI_TYPENAME = 0x000000400,
        SHGFI_ATTRIBUTES = 0x000000800,
        SHGFI_ICONLOCATION = 0x000001000,
        SHGFI_EXETYPE = 0x000002000,
        SHGFI_SYSICONINDEX = 0x000004000,
        SHGFI_LINKOVERLAY = 0x000008000,
        SHGFI_SELECTED = 0x000010000,
        SHGFI_ATTR_SPECIFIED = 0x000020000,
        SHGFI_LARGEICON = 0x000000000,
        SHGFI_SMALLICON = 0x000000001,
        SHGFI_OPENICON = 0x000000002,
        SHGFI_SHELLICONSIZE = 0x000000004,
        SHGFI_PIDL = 0x000000008,
        SHGFI_USEFILEATTRIBUTES = 0x000000010
    }

    public enum FileAttributesFlags : uint
    {
        FILE_ATTRIBUTE_ARCHIVE = 0x00000020,
        FILE_ATTRIBUTE_ENCRYPTED = 0x00004000,
        FILE_ATTRIBUTE_HIDDEN = 0x00000002,
        FILE_ATTRIBUTE_NORMAL = 0x00000080,
        FILE_ATTRIBUTE_NOT_CONTENT_INDEXED = 0x00002000,
        FILE_ATTRIBUTE_OFFLINE = 0x00001000,
        FILE_ATTRIBUTE_READONLY = 0x00000001,
        FILE_ATTRIBUTE_SYSTEM = 0x00000004,
        FILE_ATTRIBUTE_TEMPORARY = 0x00000100
    }

    public class Win32
    {
        //
        [DllImport("shell32.dll", CharSet = CharSet.Unicode, ExactSpelling = true)]
        public static extern IntPtr SHGetFileInfoW(
            string pszPath,
            FileAttributesFlags dwFileAttributes,
            ref SHFILEINFOW psfi,
            uint cbSizeFileInfo,
            ShellFileInfoFlags uFlags);
    }

    public class Win32FileInfo
    {
        public static bool GetFileInfo(string fileName, ref CItem item)
        {
            if (!System.IO.File.Exists(fileName))
                return false;
            SHFILEINFOW shinfo = new SHFILEINFOW();
            Win32.SHGetFileInfoW(
                    fileName,
                    FileAttributesFlags.FILE_ATTRIBUTE_NORMAL,
                    ref shinfo,
                    (uint)Marshal.SizeOf(shinfo),
                    ShellFileInfoFlags.SHGFI_ICON |
                    ShellFileInfoFlags.SHGFI_LARGEICON |
                    ShellFileInfoFlags.SHGFI_TYPENAME |
                    ShellFileInfoFlags.SHGFI_DISPLAYNAME
                );
            item.image = Imaging.CreateBitmapSourceFromHIcon(
                    shinfo.hIcon,
                    new Int32Rect(0, 0, 32, 32),
                    BitmapSizeOptions.FromEmptyOptions()
                );
            item.type = shinfo.szTypeName;
            item.name = shinfo.szDisplayName;
            return true;
        }
    }

	public class CItem
	{
		public BitmapSource image { get; set; }
		public string name { get; set; }
		public string type { get; set; }
	}
}